package org.implab.gradle.mercurial.tasks;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import org.gradle.api.DefaultTask;
import org.gradle.api.model.ObjectFactory;
import org.gradle.api.provider.ListProperty;
import org.gradle.api.provider.Property;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.SkipWhenEmpty;
import org.gradle.api.tasks.TaskAction;
import org.implab.gradle.Utils;
import org.implab.gradle.mercurial.MercurialExtension;

import groovy.transform.Internal;

public class HgTag extends DefaultTask {

    private final Property<String> cliCmd;

    private final Property<Boolean> force;

    private final ListProperty<String> tags;

    private String commandOutput;

    protected MercurialExtension getMercurialExtension() {
        return getProject().getExtensions().getByType(MercurialExtension.class);
    }

    @Inject
    public HgTag(ObjectFactory factory) {
        cliCmd = factory.property(String.class);
        force = factory.property(Boolean.class);
        tags = factory.listProperty(String.class);
        cliCmd.convention(getMercurialExtension().getHgCmd());

        onlyIf(t -> !getTags().get().isEmpty());
    }

    @Internal
    public Property<Boolean> getForce() {
        return force;
    }

    @Internal
    public String getCommandOutput() {
        return commandOutput;
    }

    @Input
    @SkipWhenEmpty
    public ListProperty<String> getTags() {
        return tags;
    }

    @TaskAction
    public void Run() throws IOException, InterruptedException, Exception {
        List<String> commandLine = new ArrayList<>();

        commandLine.add(cliCmd.get());

        commandLine.add("tag");

        if(force.isPresent())
            commandLine.add("-f");

        commandLine.addAll(tags.get());

        getLogger().info("Starting: {}", commandLine);

        commandOutput = Utils.execCmd(commandLine, getMercurialExtension().getOutputCharset().name());
    }

}
